"""Nessita Syntax to Docker File.

Example:

    precise:
        apt:
            apache2: 3.3-4
            bzr:
        pypi:
            foobar:
    metadata:
        test_script: ./test


will be converted to

    from ubuntu:precise
    run apt-get install apache2=3.3-4 bzr
    run mkdir /tmp/pipcache
    run pip install --download=/tmp/pipcache --no-install foobar
    run ifconfig eth0 down
    run pip install --find-links=/tmp/pipcache --no-index foobar

and also the system will use "./test" as script
"""

import yaml

# the directory where pip will store stuff
PIP_CACHE_DIR = "/tmp/pipcache"

# the conversion between our nice base names and those that are
# needed by docker
BASE_TRANSLATIONS = {
    'precise': 'ubuntu:precise',
}

def _get_base(config):
    """Process the system base stuff."""
    # support only one base for now
    if len(config) > 1:
        raise NotImplementedError("Too many base machines.")
    base = config.keys()[0]
    return ["from " + BASE_TRANSLATIONS[base]]


def _get_depends_prev(config):
    """All the dependencies before securing machine."""
    # support only one base for now
    config = config.values()[0]
    dependencies = []

    # apt
    items = config.get('apt', [])
    if items:
        parts = []
        for name, ver in sorted(items.items()):
            if ver is None:
                p = name
            else:
                p = "%s=%s" % (name, ver)
            parts.append(p)
        dep = "run apt-get install " + " ".join(parts)
        dependencies.append(dep)

    # pypi
    items = config.get('pypi', [])
    if items:
        dependencies.append("run mkdir " + PIP_CACHE_DIR)
        parts = []
        for name, ver in sorted(items.items()):
            if ver is None:
                p = name
            else:
                p = "%s==%s" % (name, ver)
            parts.append(p)
        dep = "run pip install --download=%s --no-install %s" % (
                PIP_CACHE_DIR, " ".join(parts))
        dependencies.append(dep)

    return dependencies

def _get_secure(config):
    """All that needs to be done to secure the machine."""
    return ["run ifconfig eth0 down"]

def _get_depends_post(config):
    """All the dependencies stuff after the machine was secured."""
    # support only one base for now
    config = config.values()[0]
    dependencies = []

    # pypi
    items = config.get('pypi', [])
    if items:
        parts = []
        for name, ver in sorted(items.items()):
            if ver is None:
                p = name
            else:
                p = "%s==%s" % (name, ver)
            parts.append(p)
        dep = "run pip install --find-links=%s --no-index %s" % (
                PIP_CACHE_DIR, " ".join(parts))
        dependencies.append(dep)

    return dependencies


def _get_rest(config):
    """The final stuff."""
    return []


def parse(config_text):
    """Convert Nessita Syntax and return the text for Docker."""
    config = yaml.load(config_text)
    metadata = config.pop("metadata")

    # the order for calling these functions is VERY important
    functions = [
        _get_base,
        _get_depends_prev,
        _get_secure,
        _get_depends_post,
        _get_rest,
    ]
    data = []
    for func in functions:
        items = func(config)
        data.extend(items)
    docker = "\n".join(data)

    # get stuff from the metadata
    script = metadata["test_script"]

    return script, docker
